<?php
namespace bwdosmx_OffcanvasMagic_namespace;

use bwdosmx_OffcanvasMagic_namespace\PageSettings\Page_Settings;
define( "BWDOSMX_ASFSK_ASSETS_PUBLIC_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/public" );
define( "BWDOSMX_ASFSK_ASSETS_ADMIN_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/admin" );
class Classbwdosmx_OffcanvasMagic {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function bwdosmx_admin_editor_scripts() {
		add_filter( 'script_loader_tag', [ $this, 'bwdosmx_admin_editor_scripts_as_a_module' ], 10, 2 );
	}

	public function bwdosmx_admin_editor_scripts_as_a_module( $tag, $handle ) {
		if ( 'bwdosmx_OffcanvasMagic_editor' === $handle ) {
			$tag = str_replace( '<script', '<script type="module"', $tag );
		}

		return $tag;
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/bwdosmx-offcanvas-magic-widgets.php' );
	}

	public function bwdosmx_register_widgets() {
		// Its is now safe to include Widgets files
		$this->include_widgets_files();

		// Register WidgetsF
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\bwdosmx_OffcanvasMagic_widgets());
	}

	private function add_page_settings_controls() {
		require_once( __DIR__ . '/page-settings/bwdosmx-offcanvas-magic-manager.php' );
		new Page_Settings();
	}

	// Register Category
	function bwdosmx_OffcanvasMagic_add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'bwdosmx-OffcanvasMagic-category',
			[
				'title' => esc_html__( 'Offcanvas Slide Magic', 'bwdosmx-offcanvas-magic' ),
				'icon' => 'eicon-person',
			]
		);
	}

	//css-js-link-here
	public function bwdosmx_all_assets_for_the_public(){

		wp_enqueue_script( 'bwdosmx-OffcanvasMagic-main', plugin_dir_url( __FILE__ ) . 'assets/public/js/main.js', array('jquery'), '1.3', true );

		$all_css_file = array(
			'bwdosmx-OffcanvasMagic-plugin-main' => array('bwdosmx-OffcanvasMagic_css_path_define'=>BWDOSMX_ASFSK_ASSETS_PUBLIC_DIR_FILE . '/css/main.css'),
        );
        foreach($all_css_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['bwdosmx-OffcanvasMagic_css_path_define'], null, '1.3', 'all');
        }

	}

	//admin-icon
	public function bwdosmx_all_assets_for_elementor_editor_admin(){
		$all_css_js_file = array(
            'bwdosmx-OffcanvasMagic-admin-main' => array('bwdosmx_path_admin_define'=>BWDOSMX_ASFSK_ASSETS_ADMIN_DIR_FILE . '/css/icon.css'),
        );
        foreach($all_css_js_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['bwdosmx_path_admin_define'], null, '1.3', 'all');
        }
	}

	public function __construct() {

		// For public assets
		add_action('wp_enqueue_scripts', [$this, 'bwdosmx_all_assets_for_the_public']);

		// For Elementor Editor
		add_action('elementor/editor/before_enqueue_scripts', [$this, 'bwdosmx_all_assets_for_elementor_editor_admin']);

		// Register Category
		add_action( 'elementor/elements/categories_registered', [ $this, 'bwdosmx_OffcanvasMagic_add_elementor_widget_categories' ] );

		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'bwdosmx_register_widgets' ] );

		// Register editor scripts
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'bwdosmx_admin_editor_scripts' ] );

		add_action('add_eael_progressbar_block', [$this, 'add_box_progress_bar_block'], 10, 3);

		$this->add_page_settings_controls();
	}
}

// Instantiate Plugin Class
Classbwdosmx_OffcanvasMagic::instance();